#!%%PERL%% -w
#
#  Copyright (c) 2004, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#   * Redistributions of source code must retain the above copyright notice,
#     this list of conditions and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright notice,
#     this list of conditions and the following disclaimer in the documentation
#     and/or other materials provided with the distribution.
#   * Neither the name of SWITCH nor the names of its contributors may be
#     used to endorse or promote products derived from this software without
#     specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
#  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
#  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
#  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
#  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
#  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
#  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
#  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
#  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#  POSSIBILITY OF SUCH DAMAGE.
#
#  This program is a modified version of Peter Haag's GenGraph.pl
#  in the same directory.
#  Made by Gabor Kiss <kissg@ki.ssg.iif.hu>

#BEGIN { warn "===================\n"; };
use strict;
use warnings;
use RRDs;
use Sys::Syslog; 
Sys::Syslog::setlogsock('unix');

######################################
#
# Configuration: 
# The only parameter to set:

use lib "%%LIBEXECDIR%%";

#
######################################

use NfConf;
use Log;

my $VERSION = '$Id:$';

sub SendFile {
	my $file = shift;

	my $buf;
	open(GIFILE,$file) or
		syslog 'err', "Cannot open file $file: $!", return;
	while (read(GIFILE, $buf, 8192)) {
		print $buf;
	}

} # End of SendFile


if ( !NfConf::LoadConfig() ) {
	die "$Log::ERROR\n";
}

openlog("GenGraph-hw", 'cons,pid', $NfConf::syslog_facility);

if ( scalar @ARGV != 14 ) {
	syslog 'err', "Argument error";
	syslog 'err', "Expect: profile source[:source] proto type, tstart, tend, tleft, tright, width, heigh, light";
	closelog();
	exit;
}

# Arguments
my $profile 	= shift;	# profile name
my $sources 	= shift;	# ':' separated list of sources
my $proto   	= shift;	# proto type: 'any', 'TCP', 'UDP', 'ICMP', 'other'
my $type    	= shift;	# 'flows', 'packets', 'traffic'
my $pstart		= shift;	# profile start time - UNIX format
my $tstart		= shift;	# start time - UNIX format
my $tend		= shift;	# end time - UNIX format
my $tleft   	= shift;	# left time of marker - UNIX format; 0 is no marker
my $tright  	= shift;	# right time of marker - UNIX format; 0 is no marker
my $width		= shift;	# width of graph
my $heigh		= shift;	# heigh of graph
my $light		= shift;	# light version ( small graphs ) - no title no span window

my $profiledir = "$NfConf::PROFILESTATDIR/$profile";


if ( ! -f "$profiledir/profile.dat" ) {
	syslog 'err', "Profile '$profile' unknown";
	closelog();
	exit;
}

if ( $sources eq '@EMPTY' ) {
	SendFile("$NfConf::HTMLDIR/EmptyGraph.gif");
	exit;
}

my @DEFS = split ':', $sources;

my $title = $#DEFS > 0 ? 'Source index' : '(meaningless numbers ;-)';
my $ds_extension = $proto eq 'any' ? '' : '_' . lc $proto;

my $datestr;
if ( $tleft ) {
	$datestr = scalar localtime($tleft);
} else {
	$datestr = scalar localtime($tstart) . " - " . scalar localtime($tend);
}

my $NUMDEFS = scalar @DEFS;
my $labelstep = $NUMDEFS > 50 ?
					10 : $NUMDEFS >= 15 ?
						5 : 1000 - int(1000 - $NUMDEFS/5);

my @rrdargs = ();
push @rrdargs, "-";	# output graphics to stdout
push @rrdargs, "--title", "$datestr Aberrant behavior" unless $light;
push @rrdargs, "--vertical-label", "$title" unless $light;
push @rrdargs, "--no-minor", if $light;
push @rrdargs, "--no-legend", if $light;
push @rrdargs, "--start",  "$tstart";
push @rrdargs, "--end",  "$tend";
push @rrdargs, "--width", "$width";
push @rrdargs, "--height", "$heigh";
push @rrdargs, "--lower-limit", "0";
push @rrdargs, "--upper-limit", "$NUMDEFS";
push @rrdargs, "--y-grid", "$labelstep:1";
push @rrdargs, "--rigid";
push @rrdargs, "--no-gridfit";

foreach my $def ( @DEFS ) {
	push @rrdargs, "DEF:$def=$profiledir/${def}.rrd:${type}${ds_extension}:AVERAGE";
}

# graph outside profile boundary? - grayout time before profile start
if ( !$light && ($tstart < $pstart) ) {
	push @rrdargs,
		"CDEF:outofdata=$DEFS[0],POP,TIME,$pstart,LE,1,0,IF",
		"TICK:outofdata#DDDDDD:1.0";
}

# mark time window if tleft != tright
if ( !$light && ($tright - $tleft) != 0 ) {
	push @rrdargs,
		"CDEF:background=$DEFS[0],POP,TIME,$tleft,GT,TIME,$tright,LE,*,1,0,IF",
		"TICK:background#B1FFA1:1.0";
}

for (my $i=0; $i<=$#DEFS; $i++) {
	no warnings 'once';
	my $src = $DEFS[$i];
	my $label = $NfConf::ENABLE_HW ? sprintf("%d\\:%s",$i+1,$src) : $src;
	push @rrdargs,
		"DEF:fail$src=$profiledir/${src}.rrd:${type}${ds_extension}:FAILURES",
		"CDEF:fail2$src=fail$src,fail$src,UNKN,IF",
		"LINE1:$i",
		"AREA:fail2$src$NfConf::sources{$src}{'col'}:$label:STACK",
#		"TICK:fail$src#ff0000:-0.01",
		;
}

if ( $tleft ) {
	push @rrdargs, "VRULE:$tleft#000000";
}

#warn join("\n", @rrdargs);
my ($averages,$xsize,$ysize) = RRDs::graph( @rrdargs );
	
if (my $ERROR = RRDs::error) {
	syslog 'err', "$ERROR";
#warn "$ERROR";
	syslog 'err', "Arg: '$profile', '$sources', '$proto', '$type', '$pstart', '$tstart', '$tend', '$tleft', '$tright', '$width', '$heigh', '$light'";
	foreach my $arg ( @rrdargs ) {
		syslog 'err', "RRD '$arg'";
	}
	SendFile("$NfConf::HTMLDIR/ErrorGraph.gif");
} 
closelog();

# vi: ts=4 sw=4
